# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'apertureNameDialog.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(609, 84)
        Dialog.setMinimumSize(QtCore.QSize(600, 0))
        Dialog.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.apertureNameEdit = QtWidgets.QLineEdit(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.apertureNameEdit.setFont(font)
        self.apertureNameEdit.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.apertureNameEdit.setMaxLength(80)
        self.apertureNameEdit.setObjectName("apertureNameEdit")
        self.verticalLayout.addWidget(self.apertureNameEdit)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Vertical)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 0, 1, 1, 1)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Enter desired aperture name"))
        self.label.setText(_translate("Dialog", "Enter name for aperture..."))





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'aperturesFileTagDialog.ui'
#
# Created by: PyQt5 UI code generator 5.14.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(607, 244)
        Dialog.setMinimumSize(QtCore.QSize(400, 0))
        Dialog.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.gridLayout_2 = QtWidgets.QGridLayout(Dialog)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.tagListEdit = QtWidgets.QTextEdit(Dialog)
        self.tagListEdit.setReadOnly(True)
        self.tagListEdit.setObjectName("tagListEdit")
        self.verticalLayout_2.addWidget(self.tagListEdit)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.label_3 = QtWidgets.QLabel(Dialog)
        self.label_3.setObjectName("label_3")
        self.verticalLayout.addWidget(self.label_3)
        self.apertureGroupTagEdit = QtWidgets.QLineEdit(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.apertureGroupTagEdit.setFont(font)
        self.apertureGroupTagEdit.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.apertureGroupTagEdit.setMaxLength(80)
        self.apertureGroupTagEdit.setObjectName("apertureGroupTagEdit")
        self.verticalLayout.addWidget(self.apertureGroupTagEdit)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Vertical)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setCenterButtons(True)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 0, 1, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.gridLayout_2.addLayout(self.verticalLayout_2, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Enter id to append to aperture group files"))
        self.label_2.setText(_translate("Dialog", "ids already in use:"))
        self.tagListEdit.setHtml(_translate("Dialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.AppleSystemUIFont\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.label.setText(_translate("Dialog", "new id to append (creates savedApertures-id.p, etc)"))
        self.label_3.setText(_translate("Dialog", "(Dots and dashes are converted to spaces to maintain proper format)"))



# import subprocess
# import sys
# import json, re
# import urllib.request, urllib.error, urllib.parse
# from distutils.version import StrictVersion

# 23 July 2022 It appears that the PyPI JSON API is no longer working in a way that enables
# us to find the version number of the latest pymovie in the PyPI repository, so we are
# removing it from use.
# def getMostRecentVersionOfPyMovieViaJason():
#
#     #!!!!! Many thanks to Kai Getrost for supplying this much improved version of 'getMostRecentVersionOfPymovie'
#
#     # Returns tuple of gotVersion, latestVersion
#     # (boolean and version-or-error string)
#
#     pkgName = "pymovie"
#
#     # Do a JSON request to pypi to get the latest version:
#     url = f"https://pypi.org/pypi/{pkgName}/json"
#     text = getUrlAsText(url)
#     if text is None:
#         return False, "Could not contact pypi.org to check for latest version"
#
#     # Parse the JSON result:
#     try:
#         data = json.loads(text)
#     except ValueError:
#         return False, "Could not parse JSON response from pypi.org"
#
#     # Sort versions to get the latest:
#     versions = sorted(data["releases"], key=StrictVersion, reverse=True)
#     latestVersion = versions[0]
#
#     # Ensure we have a seemingly valid version number:
#     if not re.match(r"\d+\.\d+\.\d+", latestVersion):
#         return False, f"Garbled version `{latestVersion}' from pypi.org"
#
#     # All is well, return result:
#     return True, latestVersion

# def getUrlAsText(url):
#     # Returns text string of `url', or None on error
#
#     try:
#         request = urllib.request.Request(url)
#         response = urllib.request.urlopen(request)
#     except urllib.error.URLError as exception:
#         if hasattr(exception, "reason"):
#             print(f"Fetch of `{url}' failed: {exception.reason}")
#         elif hasattr(exception, "code"):
#             print(f"Fetch of `{url}' failed: returned HTTP code {exception.code}")
#         else:
#             print(f"Fetch of `{url}' failed: Unknown reason")
#         return None
#     html = response.read()
#     return html.decode("utf-8")

def upgradePyMovie(pymovieversion):

    import subprocess

    resp = subprocess.run(['python', '-m', 'pip', 'install', '--user', '--upgrade', pymovieversion],
                          stderr=subprocess.PIPE, stdout=subprocess.PIPE)

    return resp.stdout.decode("utf-8").split('\n')


# The following function was added 23 July 2022 when (apparently) the PyPI JSON API broke.
# This uses a reliable, supported technique to get info (we are only interested in Version: )
# about a package on PyPI
# def getLatestPackageVersion(package_name: str) -> str:
#     import subprocess
#     response = subprocess.run(['python3', '-m', 'pip', 'install', f"{package_name}==0.0.0"],
#                               stderr=subprocess.PIPE, stdout=subprocess.PIPE)
#     errorResponse = response.stderr.decode("utf-8").split('\n')[0]
#
#     versions = errorResponse.split('versions: ')
#     if len(versions) == 1:  # Because the split above failed
#         # Failed to make Internet connection
#         return 'Failed to connect to PyPI - Internet connection problem?'
#     versions = versions[1].split(')')[0]  # Remove everything at and after ')'
#     latestVersion = versions.split(',')[-1].strip()
#     return latestVersion

def getLatestPackageVersion(package_name: str) -> str:
    import subprocess
    try:
        response = subprocess.run(['python', '-m', 'pip', 'install', f"{package_name}==0.0.0"],
                                  stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        errorResponse = response.stderr.decode("utf-8")
        versions = errorResponse.split('versions: ')
    except FileNotFoundError as e:
        print(f'{e}')
        versions = [0]
    if len(versions) == 1:  # Because the split above failed
        print('python not used to start pyote or no internet connection')
        try:
            response = subprocess.run(['python3', '-m', 'pip', 'install', f"{package_name}==0.0.0"],
                                      stderr=subprocess.PIPE, stdout=subprocess.PIPE)
            errorResponse = response.stderr.decode("utf-8")
            versions = errorResponse.split('versions: ')
        except FileNotFoundError as e:
            print(f'{e}')
            versions = [0]
        if len(versions) == 1:
            print('python3 not used to start pyote or no internet connection')
            try:
                response = subprocess.run(['py', '-m', 'pip', 'install', f"{package_name}==0.0.0"],
                                          stderr=subprocess.PIPE, stdout=subprocess.PIPE)
                errorResponse = response.stderr.decode("utf-8")
                versions = errorResponse.split('versions: ')
            except FileNotFoundError as e:
                print(f'{e}')
                versions = [0]
            if len(versions) == 1:
                print('py not used to start pyote or no internet connection')
                # Failed to make Internet connection
                return 'Failed to connect to PyPI - Internet connection problem?'
    versions = versions[1].split(')')[0]  # Remove everything at and after ')'
    latestVersion = versions.split(',')[-1].strip()

    return latestVersion



import pyqtgraph as pg
from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QDialog
from pymovie import apertureNameDialog
import PyQt5


class AppNameDialog(QDialog, apertureNameDialog.Ui_Dialog):
    def __init__(self):
        super(AppNameDialog, self).__init__()
        self.setupUi(self)


class HorizontalLine(pg.GraphicsObject):
    def __init__(self, rowNumber, height, width, colorStr):  # Initialize aperture specified by a bounding box
        self.pen = pg.mkPen(colorStr)
        self.y0 = rowNumber
        self.x0 = 0
        self.x1 = width
        self.h = height
        self.w = width

        # note that the use of super() is often avoided because Qt does not
        # allow to inherit from multiple QObject subclasses.
        pg.GraphicsObject.__init__(self)

    # All graphics items must have paint() defined.
    def paint(self, p, *args):
        _ = args  # Just to satisfy PyCharm code inspector (need to use args in some manner)
        p.setPen(self.pen)
        p.drawLine(self.x0, self.y0, self.x1, self.y0)

    # All graphics items must have boundingRect() defined.
    def boundingRect(self):
        return PyQt5.QtCore.QRectF(0, 0, self.w, self.h)

    def setRow(self, row):
        self.y0 = row
        self.update()


class MeasurementAperture(pg.GraphicsObject):

    def __init__(self, name, bbox, max_xpos, max_ypos):  # Initialize aperture specified by a bounding box
        self.name = name
        self.thresh = 0
        self.pen = pg.mkPen('r')
        self.color = 'red'
        self.x0, self.y0, self.xsize, self.ysize = bbox

        self.jogging_enabled = False
        self.auto_display = False
        self.thumbnail_source = False

        self.default_mask_radius = 3.0
        self.order_number = 0
        self.defaultMask = None
        self.defaultMaskPixelCount = None

        self.theta = None   # Holds angle to yellow #1 (if present)
        self.dx = None      # Holds x distance from yellow #1
        self.dy = None      # Holds y distance from yellow #1

        self.xc = None      # Will hold current x value of centroid in image coordinates
        self.yc = None      # Will hold current y value of centroid in image coordinates

        self.data = []

        # When created, we accept the callers restriction on the placement
        # of a bbox (bounding box) corner and enforce
        self.max_xpos = max_xpos
        self.max_ypos = max_ypos

        # Enforce the restrictions even during creation
        self.enforcePositioningConstraints(bbox)

        # menu creation is deferred because it is expensive (not really) and often
        # the user will never see the menu anyway.
        self.menu = None

        # note that the use of super() is often avoided because Qt does not
        # allow to inherit from multiple QObject subclasses.
        pg.GraphicsObject.__init__(self)

    def getBbox(self):
        return self.x0, self.y0, self.xsize, self.ysize

    def getCenter(self):
        return self.x0 + int(self.xsize / 2), self.y0 + int(self.ysize / 2)

    def setCenter(self, xc, yc):
        delta = int(self.xsize / 2)
        bbox = (xc - delta, yc - delta, self.xsize, self.ysize)
        self.enforcePositioningConstraints(bbox)

    def addData(self, data_tuple):
        self.data.append(data_tuple)

    def enforcePositioningConstraints(self, bbox):
        self.x0, self.y0, self.xsize, self.ysize = bbox

        # Enforce placement constraints
        if self.x0 < 0:
            self.x0 = 0
        if self.x0 > self.max_xpos:
            self.x0 = self.max_xpos

        if self.y0 < 0:
            self.y0 = 0
        if self.y0 > self.max_ypos:
            self.y0 = self.max_ypos

    def setPos(self, bbox):
        self.enforcePositioningConstraints(bbox)

    # All graphics items must have boundingRect() defined.
    def boundingRect(self):
        return PyQt5.QtCore.QRectF(self.x0, self.y0, self.xsize, self.ysize)

    # All graphics items must have paint() defined.
    def paint(self, p, *args):
        _ = args  # Just to satisfy PyCharm code inspector (need to use args in some manner)
        p.setPen(self.pen)
        p.drawRect(self.boundingRect())
        p.setPen(pg.mkPen('y'))
        p.drawLine(self.x0, self.y0, self.x0 + self.xsize, self.y0 + self.ysize)
        p.drawLine(self.x0, self.y0 + self.ysize, self.x0 + self.xsize, self.y0)

    def setGreen(self):
        self.pen = pg.mkPen('g')
        self.color = 'green'
        self.update()

    def setRed(self):
        self.pen = pg.mkPen('r')
        self.color = 'red'
        self.update()

    def setWhite(self):
        self.pen = pg.mkPen('w')
        self.color = 'white'
        self.update()

    def setYellowNoCheck(self):
        self.pen = pg.mkPen('y')
        self.color = 'yellow'
        self.update()



# These routines ares used during the manual WCS calculation

# For theory: http://spiff.rit.edu/classes/phys373/lectures/astrom/astrom.html

from numpy import cos, sin, pi, sqrt, arcsin

# The parameters to these helper functions are always dictionaries
# with the structure as shown below.

# ra and dec MUST be in degrees
# ref1 = {'ra':98.48722469, 'dec':22.33929379, 'x':170,   'y':185 }
# ref2 = {'ra':97.99874741, 'dec':22.18617554, 'x':260,   'y':393 }
# targ = {'ra':98.13950330, 'dec':22.46275600, 'x':None,  'y':None }


def cos_deg(theta_degrees):
    theta_radians = theta_degrees * pi / 180.0
    return cos(theta_radians)


def sin_deg(theta_degrees):
    theta_radians = theta_degrees * pi / 180.0
    return sin(theta_radians)


def arcsec_distance(star1, star2):
    # Returns distance between star1 and star2 in arcsec
    avg_dec = (star2['dec'] + star1['dec']) / 2.0
    delta_ra = (star2['ra'] - star1['ra']) * cos_deg(avg_dec) * 3600.0
    delta_dec = (star2['dec'] - star1['dec']) * 3600.0
    return sqrt(delta_ra ** 2 + delta_dec ** 2)


def delta_ra_arcsec(star1, star2):
    # Thanks to Michael Richmond's little simple astrometry paper
    # for pointing out the need to scale ra based on dec
    avg_dec = (star2['dec'] + star1['dec']) / 2.0  # avg_dec in degrees

    # Convert answer to arcsec as well as scale based on average dec
    delta_ra = (star2['ra'] - star1['ra']) * cos_deg(avg_dec) * 3600.0
    return delta_ra


def delta_dec_arcsec(star1, star2):
    return (star2['dec'] - star1['dec']) * 3600.0


def pixel_distance(star1, star2):
    dx = star2['x'] - star1['x']
    dy = star2['y'] - star1['y']

    return sqrt(dx * dx + dy * dy)


def angle_ra_dec(star1, star2):
    ra_delta_arcsec = delta_ra_arcsec(star1, star2)
    dec_delta_arcsec = delta_dec_arcsec(star1, star2)
    return calc_theta(ra_delta_arcsec, dec_delta_arcsec)


def angle_xy(star1, star2, xflipped, yflipped):
    delta_x_local = star2['x'] - star1['x']
    delta_y_local = star2['y'] - star1['y']

    if xflipped:
        delta_x_local = -delta_x_local
    if yflipped:
        delta_y_local = -delta_y_local

    return calc_theta(delta_x_local, delta_y_local)  # result is in degrees


def calc_theta(dx, dy):
    # This is the crucial routine for working trig problems.  It
    # returns the angle of the line from 0,0 to dx,dy no matter what
    # quadrant dx,dy may be in. The angle returned is in the range [0...360)
    # It is always positive and represents the ccw rotation from the x axis
    # toward the positive y axis.

    d = sqrt(dx * dx + dy * dy)
    if d == 0:
        return 0.0
    a = arcsin(dy / d)

    # arcsin() will not return an angle in the range[0...360), so we
    # fix that up here before returning the angle.
    if dx >= 0 and dy >= 0:
        theta = a
    elif dx <= 0 <= dy:
        theta = pi - a
    elif dx <= 0 and dy <= 0:
        theta = pi - a
    elif dx >= 0 >= dy:
        theta = pi + pi + a
    else:
        return None
    return theta * 180.0 / pi  # give answer in degrees


def convert_ra_dec_angle_to_xy(angle, ref1, ref2, xflipped, yflipped):
    offset = angle_ra_dec(ref1, ref2) - angle_xy(ref1, ref2, xflipped, yflipped)
    return angle - offset, offset


# def solve_triangle(ref1, ref2, targ, pixel_aspect_ratio, plate_scale=None, xflipped=False, yflipped=False):
#     # rescale references to account for non-square pixels
#     if pixel_aspect_ratio < 1.0:
#         ref1['x'] = pixel_aspect_ratio * ref1['x']
#         ref2['x'] = pixel_aspect_ratio * ref2['x']
#     elif pixel_aspect_ratio > 1.0:
#         ref1['y'] = ref1['y'] / pixel_aspect_ratio
#         ref2['y'] = ref2['y'] / pixel_aspect_ratio
#
#     if plate_scale is None:
#         plate_scale = arcsec_distance(ref2, ref1) / pixel_distance(ref2, ref1)
#
#     targ_theta, ra_dec_x_y_rotation = convert_ra_dec_angle_to_xy(
#         angle_ra_dec(ref1, targ), ref1, ref2, xflipped, yflipped)
#
#     d = arcsec_distance(ref1, targ) / plate_scale
#
#     ref1x = ref1['x']
#     ref1y = ref1['y']
#
#     if xflipped:
#         x_targ = -(d * cos_deg(targ_theta) - ref1x)
#     else:
#         x_targ = d * cos_deg(targ_theta) + ref1x
#
#     if yflipped:
#         y_targ = -(d * sin_deg(targ_theta) - ref1y)
#     else:
#         y_targ = d * sin_deg(targ_theta) + ref1y
#
#     # x_targ = d * cos_deg(targ_theta) + ref1['x']
#     # y_targ = d * sin_deg(targ_theta) + ref1['y']
#
#     # Compensate for pixel aspect ratio.
#     if pixel_aspect_ratio < 1.0:
#         x_targ= x_targ / pixel_aspect_ratio
#     elif pixel_aspect_ratio > 1.0:
#         y_targ = y_targ * pixel_aspect_ratio
#
#     solution = {'ra': targ['ra'], 'dec': targ['dec'], 'x': x_targ, 'y': y_targ}
#
#     return solution, plate_scale, targ_theta, ra_dec_x_y_rotation

def delta_x(star1, star2):
    return star2['x'] - star1['x']

def delta_y(star1, star2):
    return star2['y'] - star1['y']

def error(theta, a1, a2, b1, b2):
    c1 = cos(theta) * a1 - sin(theta) * a2
    c2 = sin(theta) * a1 + cos(theta) * a2
    return (c1 - b1) * (c1 - b1) + (c2 - b2) * (c2 - b2)

def hunt(theta0, dtheta, a1, a2, b1, b2):
    err0 = error(theta0, a1, a2, b1, b2)
    err1 = error(theta0 + dtheta, a1, a2, b1, b2)
    err2 = error(theta0 - dtheta, a1, a2, b1, b2)
    if err2 < err1:
        dtheta = -dtheta
    err = err0
    theta = theta0
    while True:
        theta += dtheta
        cerr = error(theta, a1, a2, b1, b2)
        if cerr > err:
            break
        err = cerr
    return theta

def align_angle(a1, a2, b1, b2):
    """Returns counter-clockwise angle to rotate (a1, a2) into (b1, b2)"""
    t = 0; dt = 0.1
    for i in range(8):
        t = hunt(t, dt, a1, a2, b1, b2)
        dt = dt / 10.0
    return t, t * 180.0 / pi

def rotate(theta, a1, a2):
    b1 = cos(theta) * a1 - sin(theta) * a2
    b2 = sin(theta) * a1 + cos(theta) * a2
    return b1, b2

def new_solve_triangle(ref1, ref2, targ, pixel_aspect_ratio, plate_scale=None):
    # rescale ref1 and ref2 to account for non-square pixels
    if pixel_aspect_ratio < 1.0:
        ref1['x'] = pixel_aspect_ratio * ref1['x']
        ref2['x'] = pixel_aspect_ratio * ref2['x']
    elif pixel_aspect_ratio > 1.0:
        ref1['y'] = ref1['y'] / pixel_aspect_ratio
        ref2['y'] = ref2['y'] / pixel_aspect_ratio

    if plate_scale is None:
        plate_scale = arcsec_distance(ref2, ref1) / pixel_distance(ref2, ref1)

    # Use ref1 and ref2 to determine rotation angle between RA/Dec and x/y systems
    a1 = delta_ra_arcsec(ref1, ref2)  / plate_scale
    a2 = delta_dec_arcsec(ref1, ref2) / plate_scale
    b1 = delta_x(ref1, ref2)
    b2 = delta_y(ref1, ref2)
    rot_radians, rot_degrees = align_angle(a1, a2, b1, b2)

    # Get pixel coords of target from RA/Dec coords of target relative to ref1

    # First: calculate pixel deltas in RA/Dec coords
    delta_t1 = delta_ra_arcsec(ref1, targ)  / plate_scale
    delta_t2 = delta_dec_arcsec(ref1, targ) / plate_scale

    # Then rotate the deltas into alignment with x/y coords
    delta_xtarg, delta_ytarg = rotate(rot_radians, delta_t1, delta_t2)

    # Apply pixel aspect ratio corrections for final report and add ref1 x/y values
    if pixel_aspect_ratio < 1.0:
        x_targ = (delta_xtarg + ref1['x']) / pixel_aspect_ratio
        y_targ = delta_ytarg + ref1['y']
    else:
        x_targ = delta_xtarg + ref1['x']
        y_targ = (delta_ytarg + ref1['y']) * pixel_aspect_ratio

    solution = {'ra': targ['ra'], 'dec': targ['dec'], 'x': x_targ, 'y': y_targ}

    return solution, plate_scale, rot_degrees
